@@ -30,8 +30,9 @@
 #    define NDEBUG
 #endif
 
-// This can be enabled to allow detection of additional integer overflows with ubsan
-//#define CHECKED
+#if defined(DEBUG) && !defined(CHECKED)
+#    define CHECKED
+#endif
 
 #include <limits.h>
 #include <stdint.h>
@@ -166,8 +167,6 @@
     }\
 }
 
-#define FF_PTR_ADD(ptr, off) ((off) ? (ptr) + (off) : (ptr))
-
 #include "libm.h"
 
 /**
@@ -355,6 +354,4 @@ void ff_check_pixfmt_descriptors(void);
  */
 int avpriv_dict_set_timestamp(AVDictionary **dict, const char *key, int64_t timestamp);
 
-extern const uint8_t ff_reverse[256];
-
 #endif /* AVUTIL_INTERNAL_H */
